#search-and-replace #toolbar {
    padding: 8px 10px;
    margin-bottom: 15px;
    border-top: 1px solid #eee;
    border-bottom: 1px solid #eee;
    background: #f8f8f8;
    color: #666;
}

#search-and-replace #toolbar input[type="text"] {
    border-radius: 4px;
    font-size: 14px;
    padding: 5px;
    color: #333;
}

#search-and-replace #toolbar input[type="text"] {
    height: 19px;
    border: 1px solid #ccc;
    padding: 2px 5px;
    margin: 0;
    vertical-align: top;
    font-size: 13px;
}

#search-and-replace #toolbar input:focus[type="text"] {
    border-color: #999;
}

#search-and-replace #toolbar input[type="submit"] {
    height: 25px;
    padding: 2px 5px;
    margin: 0;
}

#search-and-replace #toolbar form input[type="submit"]:focus,
#search-and-replace #toolbar form input[type="submit"]:hover {
    border-color: #999;
}

#search-and-replace .apply {
    background-color: #ba2121;
    color: white;
}

.search-and-replace .results {
    margin-top: 1em;
}

.results-count {
    color: #666;
    border-bottom: 1px solid #eee;
    background: #fff;
    font-size: 13px;
    padding-top: 10px;
    padding-bottom: 10px;
    line-height: 22px;
    margin: 0;
    border-top: 1px solid #ddd;  overflow: hidden;
}

mark.mark-search, mark.mark-search .mark-replace  {
    background: orange;
    color: black;
}

mark.mark-replace {
    background: lightgreen;
    color: black;
}

